package com.weface.serviceimpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.weface.code.PushResultCode;
import com.weface.common.utils.SnowIdUtil;
import com.weface.config.GeTuiApp;
import com.weface.config.GeTuiConfig;
import com.weface.dao.PushCallBackDao;
import com.weface.entity.PushCallBackEntity;
import com.weface.service.PushCallBackService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.Date;
import java.util.Map;

/**
 * @author : Administrator
 * @date : 2022/4/19 14:08
 */
@Service("pushCallBackService")
@Slf4j
@Transactional(rollbackFor = {Exception.class})
public class PushCallBackServiceImpl extends ServiceImpl<PushCallBackDao, PushCallBackEntity> implements PushCallBackService {

    @Resource
    private GeTuiApp geTuiApp;

    /**
     * 保存推送回调
     *
     * @param request 回调请求
     * @return 执行结果
     */
    @Override
    public PushResultCode saveCallBackInfo(HttpServletRequest request) {
        JSONObject jsonObject = new JSONObject();
        try {
            byte[] bytes = getRequestInputStream(request);
            String callInfo = new String(bytes);
            PushCallBackEntity callBackInfo = JSONObject.parseObject(callInfo, PushCallBackEntity.class);
            String appId = callBackInfo.getAppid();
            String str = appId + callBackInfo.getCid() + callBackInfo.getTaskid() + callBackInfo.getMsgid() + getMasterSecret(appId);
            String sign = md5Encode(str);
            if (!callBackInfo.getSign().equals(sign)) {
                return PushResultCode.error();
            }
            String taskid = callBackInfo.getTaskid();
            String cid = callBackInfo.getCid();

            callBackInfo.setId(SnowIdUtil.nextId());
            callBackInfo.setUpdateTime(new Date());
            callBackInfo.setStatus(1);
            this.save(callBackInfo);
            return PushResultCode.ok();
        } catch (IOException e) {
            log.error("处理回调函数异常:{}", e.getMessage());
            e.printStackTrace();
        }
        return PushResultCode.error();
    }

    private String getMasterSecret(String appId) {
        String masterSecret = null;
        Map<String, GeTuiConfig> apps = geTuiApp.getApps();
        for (Map.Entry<String, GeTuiConfig> entry : apps.entrySet()) {
            if (appId.equals(entry.getValue().getAppId())) {
                masterSecret = entry.getValue().getMastersecret();
                break;
            }
        }
        Assert.notNull(masterSecret, "密钥不能为空");
        return masterSecret;
    }

    public static String md5Encode(String str) {
        MessageDigest md = DigestUtils.getMd5Digest();
        return Arrays.toString(md.digest(str.getBytes(StandardCharsets.UTF_8)));
    }

    public static byte[] getRequestInputStream(HttpServletRequest request) throws IOException {
        ServletInputStream input = request.getInputStream();
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int position = 0;
        while (true) {
            position = input.read(buffer);
            if (position == -1) {
                break;
            }
            output.write(buffer, 0, position);
        }
        return output.toByteArray();

    }
}
